/**
 * Easy Coverage for Java
 * <p>
 * Copyright 2011-2012 Tocea Technology Solutions http://www.tocea.com
 * <p>
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Lesser General Public License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 * <p>
 **/

package com.tocea.easycoverage.framework;



import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.annotations.SkipAutomaticTests;
import com.tocea.easycoverage.framework.behaviour.api.IClassTest;
import com.tocea.easycoverage.framework.behaviour.api.ITestBehaviour;
import com.tocea.easycoverage.framework.unit.WeakImplementationException;



/**
 * This type describes the test harness.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 12 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class TestHarness
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER          = LoggerFactory.getLogger(TestHarness.class);
    
    private final List<ITestBehaviour>    testBehaviours  = new ArrayList<ITestBehaviour>();
    
    private Set<Class<?>>                 classes         = new HashSet<Class<?>>();
    
    private Set<Object>                   objects         = new HashSet<Object>();
    
    
    private int                           failedInstances = 0;
    
    
    private int                           failedClasses   = 0;
    
    
    
    /**
     * This method performs the addition of a new behaviour to process on the class to test.
     * 
     * @param _newInstance
     *            the new instance.
     */
    public void addBehaviour(final ITestBehaviour _newInstance) {
    
    
        if (_newInstance == null) {
            throw new IllegalArgumentException();
        }
        this.testBehaviours.add(_newInstance);
    }
    
    
    /**
     * This method performs the processing of all tests.
     * 
     * @return the processing of all tests.
     */
    public boolean processTests() {
    
    
        boolean res = true;
        LOGGER.debug("Initialization of the tests");
        res &= this.testingInstances();
        res &= this.testingClasses();
        LOGGER.info("Tests finished", this.failedInstances, this.failedClasses);
        LOGGER.info("\tInstance failed {} on {}", this.failedInstances, this.objects.size());
        LOGGER.info("\tClasses failed {} on {}", this.failedClasses, this.classes.size());
        return res;
        
    }
    
    
    /**
     * This method sets the classes to analysis.
     * 
     * @param _setClasses
     *            the classes to analysis.
     */
    public void setClasses(final Set<Class<?>> _setClasses) {
    
    
        if (_setClasses == null) {
            throw new IllegalArgumentException();
        }
        
        this.classes = _setClasses;
    }
    
    
    /**
     * This method sets the classes to analysis.
     * 
     * @param _setObjects
     *            the classes to analysis.
     */
    public void setObjects(final Set<Object> _setObjects) {
    
    
        if (_setObjects == null) {
            throw new IllegalArgumentException();
        }
        this.objects = _setObjects;
        
        
    }
    
    
    /**
     * This method performs the test of a class
     * 
     * @param _class
     * @return the test of a class
     */
    private boolean testingClass(final Class<?> _class) {
    
    
        boolean res = true;
        if (_class.isAnnotationPresent(SkipAutomaticTests.class)) {
            LOGGER.info("\t[SKIP] {}", _class);
            
            return true;
        }
        LOGGER.info("\t[TEST] {}", _class);
        for (final ITestBehaviour testBehaviour : this.testBehaviours) {
            final IClassTest newInstanceTest = testBehaviour.newTest(_class);
            if (newInstanceTest == null) {
                continue;
            }
            try {
                newInstanceTest.performTest();
            } catch (final WeakImplementationException _exception) {
                res = false;
                LOGGER.error("Failed execution of a unit test {}", _exception);
            }
            
        }
        return res;
    }
    
    
    /**
     * This method performs the testing classes.
     * 
     * @return the result of tests
     */
    private boolean testingClasses() {
    
    
        LOGGER.info("Testing the classes");
        
        
        boolean res = true;
        LOGGER.debug("{} classes to test; ", this.classes.size());
        for (final Class<?> clazz : this.classes) {
            final boolean individualRes = this.testingClass(clazz);
            if (!individualRes) {
                LOGGER.error("\t ->[FAILED]");
                this.failedClasses++;
            }
            res &= individualRes;
        }
        return res;
        
    }
    
    
    /**
     * This method performs the test of an instance.
     * 
     * @param _object
     *            the result of the test.
     */
    private boolean testingInstanceObject(final Object _object) {
    
    
        boolean res = true;
        LOGGER.info("\t[TEST] Object '{}' of {}", _object, _object.getClass());
        for (final ITestBehaviour testBehaviour : this.testBehaviours) {
            final IClassTest newInstanceTest = testBehaviour.newInstanceTest(_object);
            if (newInstanceTest == null) {
                continue;
            }
            try {
                newInstanceTest.performTest();
            } catch (final WeakImplementationException _exception) {
                res = false;
                LOGGER.error("Failed execution of a unit test {}", _exception);
            }
            
        }
        return res;
        
    }
    
    
    /**
     * This method performs the testing classes.
     * 
     * @return the result of tests
     */
    private boolean testingInstances() {
    
    
        LOGGER.info("Testing the instance of objects");
        
        boolean res = true;
        LOGGER.debug("{} objects to test; ", this.objects.size());
        for (final Object object : this.objects) {
            final boolean individualRes = this.testingInstanceObject(object);
            if (!individualRes) {
                LOGGER.error("\t ->[FAILED]");
                this.failedInstances++;
            }
            res &= individualRes;
        }
        return res;
    }
}
